import os
import requests
from common.version_helpers import get_split_version

LAST_RELEASE_VERSION = "LAST_RELEASE_VERSION"
LAST_RELEASE_HASH = "LAST_RELEASE_HASH"
BRANCH_LAST_RELEASE_HASH = "{}_LAST_RELEASE_HASH".format(os.environ['CI_COMMIT_BRANCH']).upper()

# Присвоение значения переменной проекта
def set_value(api_token, project_id, key, value):
    url = "https://gitlab.evosoft.xyz/api/v4/projects/{}/variables/{}".format(project_id, key)

    payload='key={}&value={}&protected=false&masked=false'.format(key, value)

    headers = {
        'PRIVATE-TOKEN': api_token,
        'Content-Type': 'application/x-www-form-urlencoded'
    }

    response = requests.put(url, headers=headers, data=payload)

    if response.status_code != 200:
        create_value(api_token, project_id, key, value)
    else:
        print("Обновлен ключ {} со значением {}.".format(key, value))

# Создание переменной проекта
def create_value(api_token, project_id, key, value):
    url = "https://gitlab.evosoft.xyz/api/v4/projects/{}/variables/".format(project_id)

    payload='key={}&value={}&protected=false&masked=false'.format(key, value)

    headers = {
        'PRIVATE-TOKEN': api_token,
        'Content-Type': 'application/x-www-form-urlencoded'
    }

    response = requests.post(url, headers=headers, data=payload)

    if response.status_code != 201:
        raise Exception("Не удалось создать ключ {}".format(key))
    else:
        print("Создан ключ {} со значением {}.".format(key, value))

# получение значения переменной проекта
def get_value(api_token, project_id, key, default_value):
    url = "https://gitlab.evosoft.xyz/api/v4/projects/{}/variables/{}".format(project_id, key)

    # glpat-6uYCYZQS9jNnbrhmn4nn
    headers = {
        'PRIVATE-TOKEN': api_token
    }

    response = requests.get(url, headers=headers)

    if response.status_code == 200:
        value = response.json()['value']
        print("Получен ключ {} со значением {}.".format(key, value))
        return value
    if response.status_code == 404:
        create_value(api_token, project_id, key, default_value)
        print("Получен ключ {} со значением {}.".format(key, default_value))
        return default_value
    else:
        raise Exception("Не удалось получить данные о ключе {}".format(key))

# установка релизных переменных проекта
def set_release_values(api_token, project_id, new_version, new_hash):
    set_value(api_token, project_id, LAST_RELEASE_VERSION, new_version)
    set_value(api_token, project_id, BRANCH_LAST_RELEASE_HASH, new_hash)

# Создание gitlab релиза
def create_release(api_token, project_id, version):
    url = "https://gitlab.evosoft.xyz/api/v4/projects/{}/releases".format(project_id)

    data = {
        'tag_name': version
    }
    
    headers = {
        'PRIVATE-TOKEN': api_token,
        'Content-Type': 'application/x-www-form-urlencoded'
    }

    requests.post(url, headers=headers, data=data)

# Получение информации о последнем релизе
def get_last_release(api_token, project_id):
    version = get_value(api_token, project_id, LAST_RELEASE_VERSION, "0.1.0")
    hexsha = get_value(api_token, project_id, BRANCH_LAST_RELEASE_HASH, ".")

    return ( get_split_version(version), hexsha )

# Создание gitlab тега
def create_tag(api_token, project_id, branch_name, version, commits):
    message = ""
    for commit in commits:
        message = "{} [{}] {}\n{}".format(commit['timestamp'], commit['author_email'], commit['message'], message)

    url = "https://gitlab.evosoft.xyz/api/v4/projects/{}/repository/tags?tag_name={}&ref={}&message={}".format(project_id, version, branch_name, message)

    headers = {
        'PRIVATE-TOKEN': api_token,
        'Content-Type': 'application/x-www-form-urlencoded'
    }

    requests.post(url, headers=headers)
